@@ -413,6 +413,28 @@ read_conf(unsigned int prog)
       SENDER_INACTIVE_EXPIRE=extract_seconds(extract_array[0]);
     }
 
+    /* SENDER_THROTTLE_AUTOBLACKLIST */
+    if(strncmp(confbuf, "SENDER_THROTTLE_AUTOBLACKLIST=", 30) == 0)
+    { 
+      extract(0, confbuf, 29);
+      SENDER_THROTTLE_AUTOBLACKLIST=atol(extract_array[0]);
+    }
+    
+    /* SENDER_THROTTLE_AUTOBLACKLIST_NUMBER */
+    if(strncmp(confbuf, "SENDER_THROTTLE_AUTOBLACKLIST_NUMBER=", 37) == 0)
+    { 
+      extract(0, confbuf, 36);
+      SENDER_THROTTLE_AUTOBLACKLIST_NUMBER=atol(extract_array[0]);
+    }
+
+    /* SENDER_THROTTLE_AUTOBLACKLIST_EXPIRE */
+    if(strncmp(confbuf, "SENDER_THROTTLE_AUTOBLACKLIST_EXPIRE=", 37) == 0)
+    { 
+      extract(0, confbuf, 36);
+      SENDER_THROTTLE_AUTOBLACKLIST_EXPIRE=extract_seconds(extract_array[0]);
+    }
+
+    
     /* RECIPIENTTHROTTLE */
     if(strncmp(confbuf, "RECIPIENTTHROTTLE=", 18) == 0)
     { 
@@ -768,6 +790,9 @@ read_conf(unsigned int prog)
     logmessage("config: sender timelimit> %d\n", SENDERTIMELIMIT);
     logmessage("config: sender msgsize> %d\n", SENDERMSGSIZE);
     logmessage("config: sender expire inactive> %d\n", SENDER_INACTIVE_EXPIRE);
+    logmessage("config: sender throttle autoblacklisting> %d\n", SENDER_THROTTLE_AUTOBLACKLIST);
+    logmessage("config: sender throttle autoblacklist number> %d\n", SENDER_THROTTLE_AUTOBLACKLIST_NUMBER);
+    logmessage("config: sender throttle autoblacklist expire> %d\n", SENDER_THROTTLE_AUTOBLACKLIST_EXPIRE);
     logmessage("\n");
 
     logmessage(" ---- RECIPIENT THROTTLE CONFIG ----\n");
@@ -830,7 +855,10 @@ logmessage(const char *fmt, ...)
   va_start(ap, fmt);
   
   if(DAEMON == 0)
+  {
     vfprintf(stdout, fmt, ap);
+    fflush(stdout);
+  }
   else
     vsyslog(SYSLOG_FACILITY, fmt, ap);
   
@@ -1159,90 +1187,79 @@ drop_privs(void)
  *  purpose: reply/talk to Postfix
  *   return: 0=sucessfull write(), 1=failed write()
  */
-int
+void
 policy_reply(unsigned int fd, int code, int status)
 {
   /* keep gcc quiet for now */
   status = 0;
-  
-  /* accept: always allow */
-  if(code == 0)
+ 
+
+  switch (code)
   {
-    
-    /* dump greylisting information into mail? */
-    if((GREYLISTING==1) && (GREYLIST_X_HEADER==1))
-    {
+    /* accept: always allow */
+    case 0:
+      /* dump greylisting information into mail? */
+      if((GREYLISTING==1) && (GREYLIST_X_HEADER==1))
+      {
       
-      /* whitelisted */
-      snprintf(xgreylist_array[fd], 128, "%s host: %s\n\n",
+        /* whitelisted */
+        snprintf(xgreylist_array[fd], 128, "%s host: %s\n\n",
         POSTFIX_X_HEADER, host_array[fd][2]);
       
-      /* if not whitelisted, greylist.c already filled in all the details */
-      w_write(fd, xgreylist_array[fd], strlen(xgreylist_array[fd]));
-      return (1);
-    }
-    
-    if(w_write(fd, POSTFIX_GOOD, strlen(POSTFIX_GOOD)) == -1)
-      return (0);
-  }
+        /* if not whitelisted, greylist.c already filled in all the details */
+        buf_write(fd, xgreylist_array[fd], strlen(xgreylist_array[fd]));
+      } else {
+    	buf_write(fd, POSTFIX_GOOD, strlen(POSTFIX_GOOD));
+      }
+
+    break;
 
   /* reject: greylisting */
-  if(code == -1)
-  {
-    if(w_write(fd, postfix_greylist, strlen(postfix_greylist)) == -1)
-      return (0);
-  }
+  case -1:
+    buf_write(fd, postfix_greylist, strlen(postfix_greylist));
+    break;
 
   /* reject: blacklisted */
-  if(code == -2)
-  {
-    if(w_write(fd, postfix_blacklist, strlen(postfix_blacklist)) == -1)
-      return (0);
-  }
+  case -2:
+    buf_write(fd, postfix_blacklist, strlen(postfix_blacklist));
+    break;
 
   /* reject: message size too big */
-  if(code == -3)
-  {
-    if(w_write(fd, postfix_bad_size, strlen(postfix_bad_size)) == -1)
-      return (0);
-  }
-
+  case -3:
+    buf_write(fd, postfix_bad_size, strlen(postfix_bad_size));
+    break;
+    
   /* reject: spam trap address */
-  if(code == -4)
-  {
-    if(w_write(fd, postfix_spamtrap, strlen(postfix_spamtrap)) == -1)
-      return (0);
-  }
-  
+  case -4:
+    buf_write(fd, postfix_spamtrap, strlen(postfix_spamtrap));
+    break;
+    
   /* reject: max sender quota exceeded */
-  if(code == -5)
-  {
-    if(w_write(fd, postfix_sender_quota_exceeded, strlen(postfix_sender_quota_exceeded)) == -1)
-      return (0);
-  }
-
+  case -5:
+    buf_write(fd, postfix_sender_quota_exceeded, strlen(postfix_sender_quota_exceeded));
+    break;
+    
   /* reject: helo checking */
-  if(code == -6)
-  {
-    if(w_write(fd, postfix_blacklist, strlen(postfix_blacklist)) == -1)
-      return (0);
-  }
-
+  case -6:
+    buf_write(fd, postfix_blacklist, strlen(postfix_blacklist));
+    break;
+    
   /* reject: max recipient quota exceeded */
-  if(code == -7)
-  {
-    if(w_write(fd, postfix_recipient_quota_exceeded, strlen(postfix_recipient_quota_exceeded)) == -1)
-      return (0);
-  }
-
+  case -7:
+    buf_write(fd, postfix_recipient_quota_exceeded, strlen(postfix_recipient_quota_exceeded));
+    break;
+    
   /* reject: max recipient quota exceeded */
-  if(code == -8)
-  {
-    if(w_write(fd, POSTFIX_MODULE_FAILURE, strlen(POSTFIX_MODULE_FAILURE)) == -1)
-      return (0);
-  }
+  case -8:
+    buf_write(fd, POSTFIX_MODULE_FAILURE, strlen(POSTFIX_MODULE_FAILURE));
+    break;
 
-  return (1); /* write() failed */
+  /* something bad happened */
+  default:
+    logmessage("WARNING: policy_reply called with unknown code\n");
+    buf_write(fd, POSTFIX_MODULE_FAILURE, strlen(POSTFIX_MODULE_FAILURE));
+    break;
+  }
 }
 
 
@@ -1260,7 +1277,7 @@ db_failure(unsigned int fd, char *module)
   if(FAILSAFE==1)
   {
     /* do not fail */
-    logmessage("rcpt=%lu, %s=pass, host=%s (%s), from=%s, to=%s, size=%s\n",
+    logmessage("rcpt=%lu, %s=bypass, host=%s (%s), from=%s, to=%s, size=%s\n",
       rcpt_count,               /* recipient count      */
       module,                   /* module name          */
       host_array[fd][2],        /* host                 */
@@ -1270,6 +1287,7 @@ db_failure(unsigned int fd, char *module)
       triplet_array[fd][3]      /* size                 */
     );  
 
+    mysql_failure_count++;
     policy_reply(fd, 0, 0);
     return (-20); 
   }             
@@ -1277,7 +1295,7 @@ db_failure(unsigned int fd, char *module)
   if(FAILSAFE==0)
   {             
     /* fail as requested */
-    logmessage("rcpt=%lu, %s=fail, host=%s (%s), from=%s, to=%s, size=%s\n",
+    logmessage("rcpt=%lu, %s=failed, host=%s (%s), from=%s, to=%s, size=%s\n",
       rcpt_count,               /* recipient count      */
       module,                   /* module name          */
       host_array[fd][2],        /* host                 */
